
module Orgy
  # === OrgObject
  # OrgObject provides a basic set of functionality for org-mode elements that define some sort of structure
  # If any org-element is capable of having child elements, it should extend this class.
  class OrgObject
    include Enumerable

    public

    # Hash that contains data relevant to the object
    attr_accessor :data 

    # Hash that contains attributes of object
    attr_accessor :attribute

    # Array of children for object
    attr_accessor :children 

    # String containing the indent delimiter
    # By default it is set to two spaces. However, you can change this to use tabs like so: 
    #     OrgObject.indent_delimiter = "\t"
    attr_accessor :indent_delimiter

    # Create new instance of OrgObject
    def initialize
      @data = Hash.new
      @clock = []
      @children = []
      @attribute = { 
                      :type => self.class, 
                      :clock => @clock 
                    }
      
      if self.class == Orgy::OrgObject then
        @indent = 0
      else
        @indent = 1
      end
      @indent_delimiter = "  "
    end

    # Add child element into children array
    def add_child child
      @children << child
    end

    # Delete child element that matches argument
    def delete_child child
      @children.delete child
    end

    # Push a child element into children array
    def push_child child 
      @children.push child
    end

    # Pop a child element into children array
    def pop_child
      @children.pop
    end

    # Tests to see if self contains children
    def contains_children?
      if @children == nil then false else true end
    end

    # Iterates through each child in array.
    # Also allows for current any object that extends OrgObject to be enumerable
    def each(&block)
      @children.each do | child |
        block.call(child)
      end
    end

    # Print children to string
    def to_s
      str = ""
      if schedule? then str << "\n" << print_schedule end
      if deadline? then str << "\n" << print_deadline end
      @children.each do | child |
        child.set_indent @indent+1
        str << child.to_s
      end
      return str
    end

    # Schedule element for a time
    def schedule org_timestamp
      if not org_timestamp.class == Orgy::OrgTimeStamp then
        raise ArgumentError, "Argument must be a kind of Orgy::OrgTimeStamp"
      end
      @attribute[:scheduled] = org_timestamp
    end

    # Unschedule element
    def clear_schedule
      @attribute[:scheduled] = nil 
    end

    # Tests if element is scheduled
    def schedule?
      @attribute[:scheduled] != nil
    end

    # Mark a deadline on element
    def deadline org_timestamp
      if not org_timestamp.class == Orgy::OrgTimeStamp then
        raise ArgumentError, "Argument must be a kind of Orgy::OrgTimeStamp"
      end
      @attribute[:deadline] = org_timestamp
    end

    # Clear a deadline
    def clear_deadline
      @attribute[:deadline] = nil
    end

    # Tests if deadline exists
    def deadline?
      @attribute[:deadline] != nil
    end

    protected
    # Set intentation of an object
    # It's probably best not to call this directly.
    def set_indent num
      @indent = num
    end

    def print_schedule
      str = ""
      if schedule? then
        str << 'SCHEDULED: ' << @attribute[:scheduled].to_s << "\n"
      end
      str
    end

    def print_deadline
      str = ""
      if deadline? then
        str << 'DEADLINE: ' << @attribute[:deadline].to_s << "\n"
      end
      str
    end
    
  end
end
